import { PostgrestHasMorePaginationResponse, PostgrestPaginationResponse, QueryWithoutWildcard, PostgrestHasMorePaginationCacheData, AnyPostgrestResponse } from '@supabase-cache-helpers/postgrest-shared';
export * from '@supabase-cache-helpers/postgrest-shared';
import * as _supabase_cache_helpers_postgrest_filter from '@supabase-cache-helpers/postgrest-filter';
import { PostgrestParser, PostgrestQueryParserOptions, PostgrestFilter } from '@supabase-cache-helpers/postgrest-filter';
export * from '@supabase-cache-helpers/postgrest-filter';
import { InsertFetcherOptions, UpdateFetcherOptions, UpsertFetcherOptions, DeleteFetcherOptions } from '@supabase-cache-helpers/postgrest-fetcher';
export * from '@supabase-cache-helpers/postgrest-fetcher';
import { DeleteItemProps, UpsertItemProps, DecodedKey, PostgrestMutatorOpts } from '@supabase-cache-helpers/postgrest-mutate';
import { PostgrestTransformBuilder, PostgrestQueryBuilder, PostgrestError as PostgrestError$1, PostgrestResponse, PostgrestSingleResponse, PostgrestMaybeSingleResponse } from '@supabase/postgrest-js';
import { GenericSchema, GenericTable } from '@supabase/postgrest-js/dist/module/types';
import { Key, Middleware, MutatorOptions, SWRResponse, SWRConfiguration } from 'swr';
import * as swr_infinite from 'swr/infinite';
import { SWRInfiniteHook, SWRInfiniteKeyLoader, SWRInfiniteFetcher, SWRInfiniteConfiguration, SWRInfiniteResponse } from 'swr/infinite';
import { GetResult } from '@supabase/postgrest-js/dist/module/select-query-parser';
import { PostgrestError, RealtimePostgresChangesPayload, SupabaseClient, RealtimePostgresChangesFilter, REALTIME_POSTGRES_CHANGES_LISTEN_EVENT, RealtimeChannel } from '@supabase/supabase-js';
export { PostgrestError } from '@supabase/supabase-js';
import { SWRMutationConfiguration, SWRMutationResponse } from 'swr/mutation';
export { SWRMutationConfiguration } from 'swr/mutation';

/**
 * Returns a function that can be used to delete an item into the SWR cache.
 * This hook does not make any HTTP requests and is intended to be used for custom cache updates.
 *
 * @param opts - Options for the delete operation, excluding the input record.
 *
 * @returns A function that takes a record of type `Type` and returns a promise that resolves once the record has been deleted from the cache.
 * **/
declare function useDeleteItem<Type extends Record<string, unknown>>(opts: Omit<DeleteItemProps<Type>, 'input'>): (input: Type) => Promise<void>;

/**
 * Returns a function that can be used to upsert an item into the SWR cache.
 * This hook does not make any HTTP requests and is intended to be used for custom cache updates.
 *
 * @param opts - Options for the upsert operation, excluding the input record.
 *
 * @returns A function that takes a record of type `Type` and returns a promise that resolves once the record has been upserted into the cache.
 * **/
declare function useUpsertItem<Type extends Record<string, unknown>>(opts: Omit<UpsertItemProps<Type>, 'input'>): (input: Type) => Promise<void>;

declare const KEY_PREFIX = "postgrest";
declare const POSTGREST_FILTER_KEY_PREFIX = "postgrest-filter";
declare const KEY_SEPARATOR = "$";
declare const INFINITE_PREFIX = "$inf$";
declare const INFINITE_KEY_PREFIX = "page";

declare const createKeyGetter: <Schema extends GenericSchema, Table extends Record<string, unknown>, Result>(query: PostgrestTransformBuilder<Schema, Table, Result, unknown> | null, pageSize: number) => (pageIndex: number, previousPageData: (PostgrestHasMorePaginationResponse<Result> | PostgrestPaginationResponse<Result>)[]) => PostgrestTransformBuilder<Schema, Table, Result, unknown> | null;

type DecodedSWRKey = DecodedKey & {
    isInfinite: boolean;
    key: string;
    isInfiniteKey: boolean;
};

declare const decode: (key: Key) => DecodedSWRKey | null;

declare const encode: <Result>(parser: PostgrestParser<Result>, isInfinite: boolean) => string;

declare const middleware: Middleware;
declare const infiniteMiddleware: <Result>(useSWRInfiniteNext: SWRInfiniteHook) => (keyFn: SWRInfiniteKeyLoader, fetcher: SWRInfiniteFetcher, config: SWRInfiniteConfiguration) => swr_infinite.SWRInfiniteResponse<any, any>;

declare const usePostgrestFilterCache: <R extends Record<string, unknown>>() => (query: string, opts?: PostgrestQueryParserOptions) => PostgrestFilter<any>;

declare const useQueriesForTableLoader: (table: string) => () => {
    paths: _supabase_cache_helpers_postgrest_filter.Path[];
    filters: _supabase_cache_helpers_postgrest_filter.FilterDefinitions;
}[];

type Operation = 'Insert' | 'UpdateOne' | 'Upsert' | 'DeleteOne';
type GetFetcherOptions<S extends GenericSchema, T extends GenericTable, O extends Operation> = O extends 'Insert' ? InsertFetcherOptions<S, T> : O extends 'UpdateOne' ? UpdateFetcherOptions<S, T> : O extends 'Upsert' ? UpsertFetcherOptions<S, T> : O extends 'DeleteOne' ? DeleteFetcherOptions<S, T> : never;
type GetInputType<T extends GenericTable, O extends Operation> = O extends 'DeleteOne' ? Partial<T['Row']> : O extends 'Insert' | 'Upsert' ? T['Insert'][] : O extends 'UpdateOne' ? T['Update'] : never;
type GetReturnType<S extends GenericSchema, T extends GenericTable, Relationships, O extends Operation, Q extends string = '*', R = GetResult<S, T['Row'], Relationships, Q extends '*' ? '*' : Q>> = O extends 'UpdateOne' ? R | null : O extends 'DeleteOne' ? R | null : O extends 'Insert' | 'Upsert' ? R[] | null : never;
type UsePostgrestSWRMutationOpts<S extends GenericSchema, T extends GenericTable, Relationships, O extends Operation, Q extends string = '*', R = GetResult<S, T['Row'], Relationships, Q extends '*' ? '*' : Q>> = PostgrestMutatorOpts<T['Row']> & Pick<MutatorOptions, 'throwOnError' | 'revalidate'> & SWRMutationConfiguration<GetReturnType<S, T, Relationships, O, Q, R>, PostgrestError, GetInputType<T, O>, string> & {
    disableAutoQuery?: boolean;
} & GetFetcherOptions<S, T, O>;

/**
 * Hook for performing a DELETE mutation on a PostgREST resource.
 *
 * @param qb - The PostgrestQueryBuilder instance for the resource.
 * @param primaryKeys - An array of primary key column names for the table.
 * @param query - An optional query string.
 * @param opts - An optional object of options to configure the mutation.
 * @returns A SWRMutationResponse object containing the mutation response data, error, and mutation function.
 */
declare function useDeleteMutation<S extends GenericSchema, T extends GenericTable, Relationships, Q extends string = '*', R = GetResult<S, T['Row'], Relationships, Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T['Row'])[], query?: QueryWithoutWildcard<Q> | null, opts?: UsePostgrestSWRMutationOpts<S, T, Relationships, 'DeleteOne', Q, R>): SWRMutationResponse<R | null, PostgrestError$1, Partial<T['Row']>>;

/**
 * Hook for performing an INSERT mutation on a PostgREST resource.
 *
 * @param qb - The PostgrestQueryBuilder instance for the resource.
 * @param primaryKeys - An array of primary key column names for the table.
 * @param query - An optional query string.
 * @param opts - An optional object of options to configure the mutation.
 * @returns A SWRMutationResponse object containing the mutation response data, error, and mutation function.
 */
declare function useInsertMutation<S extends GenericSchema, T extends GenericTable, Relationships, Q extends string = '*', R = GetResult<S, T['Row'], Relationships, Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T['Row'])[], query?: QueryWithoutWildcard<Q> | null, opts?: UsePostgrestSWRMutationOpts<S, T, Relationships, 'Insert', Q, R>): SWRMutationResponse<R[] | null, PostgrestError$1, T['Insert'][]>;

/**
 * Hook for performing an UPDATE mutation on a PostgREST resource.
 *
 * @param qb - The PostgrestQueryBuilder instance for the resource.
 * @param primaryKeys - An array of primary key column names for the table.
 * @param query - An optional query string.
 * @param opts - An optional object of options to configure the mutation.
 * @returns A SWRMutationResponse object containing the mutation response data, error, and mutation function.
 */
declare function useUpdateMutation<S extends GenericSchema, T extends GenericTable, Relationships, Q extends string = '*', R = GetResult<S, T['Row'], Relationships, Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T['Row'])[], query?: QueryWithoutWildcard<Q> | null, opts?: UsePostgrestSWRMutationOpts<S, T, Relationships, 'UpdateOne', Q, R>): SWRMutationResponse<R | null, PostgrestError$1, T['Update']>;

/**
 * Hook for performing an UPSERT mutation on a PostgREST resource.
 *
 * @param qb - The PostgrestQueryBuilder instance for the resource.
 * @param primaryKeys - An array of primary key column names for the table.
 * @param query - An optional query string.
 * @param opts - An optional object of options to configure the mutation.
 * @returns A SWRMutationResponse object containing the mutation response data, error, and mutation function.
 */
declare function useUpsertMutation<S extends GenericSchema, T extends GenericTable, Relationships, Q extends string = '*', R = GetResult<S, T['Row'], Relationships, Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T['Row'])[], query?: QueryWithoutWildcard<Q> | null, opts?: UsePostgrestSWRMutationOpts<S, T, Relationships, 'Upsert', Q, R>): SWRMutationResponse<R[] | null, PostgrestError$1, T['Insert'][], string>;

/**
 * The return type of the `useInfiniteQuery` hook
 */
type UseInfiniteQueryReturn<Result extends Record<string, unknown>> = SWRInfiniteResponse<Exclude<PostgrestResponse<Result>['data'], null>, PostgrestError$1>;
/**
 * A hook to perform an infinite postgrest query
 * @param query The postgrest query builder
 * @param config Optional SWRInfiniteConfiguration options to configure the hook
 * @returns An object containing the query results and other SWR-related properties
 */
declare function useInfiniteQuery<Schema extends GenericSchema, Table extends Record<string, unknown>, Result extends Record<string, unknown>>(query: PostgrestTransformBuilder<Schema, Table, Result[]> | null, config?: SWRInfiniteConfiguration & {
    pageSize?: number;
}): UseInfiniteQueryReturn<Result>;

type SWRInfiniteScrollPostgrestResponse<Result> = Omit<SWRInfiniteResponse<PostgrestHasMorePaginationCacheData<Result>, PostgrestError$1>, 'data'> & {
    loadMore: null | (() => void);
    data: Result[] | undefined;
};
/**
 * The return value of useInfiniteScrollQuery hook.
 */
type UseInfiniteScrollQueryReturn<Result extends Record<string, unknown>> = Omit<SWRInfiniteResponse<PostgrestHasMorePaginationResponse<Result>, PostgrestError$1>, 'data'> & {
    loadMore: null | (() => void);
    data: Result[] | undefined;
};
/**
 * A hook that provides infinite scroll capabilities to PostgREST queries using SWR.
 *
 * @param {PostgrestTransformBuilder<Schema, Table, Result[]> | null} query - The PostgREST query.
 * @param {SWRInfiniteConfiguration & { pageSize?: number }} [config] - The SWRInfinite configuration.
 * @returns {UseInfiniteScrollQueryReturn<Result>} - The infinite scroll query result.
 */
declare function useInfiniteScrollQuery<Schema extends GenericSchema, Table extends Record<string, unknown>, Result extends Record<string, unknown>>(query: PostgrestTransformBuilder<Schema, Table, Result[]> | null, config?: SWRInfiniteConfiguration & {
    pageSize?: number;
}): UseInfiniteScrollQueryReturn<Result>;

type SWRInfinitePaginationPostgrestResponse<Result> = Omit<SWRInfiniteResponse<PostgrestHasMorePaginationResponse<Result>, PostgrestError$1>, 'data' | 'size' | 'setSize'> & {
    pages: SWRInfiniteResponse<Result[], PostgrestError$1>['data'];
    currentPage: null | Result[];
    pageIndex: number;
    setPage: (idx: number) => void;
    nextPage: null | (() => void);
    previousPage: null | (() => void);
};
/**
 * The return value of the `usePaginationQuery` hook.
 */
type UsePaginationQueryReturn<Result extends Record<string, unknown>> = SWRInfinitePaginationPostgrestResponse<Result>;
/**
 * A hook for paginating through a PostgREST response.
 *
 * @param query - A PostgREST query builder.
 * @param config - A SWR configuration object.
 * @param config.pageSize - The number of items per page.
 * @returns An object containing the paginated data and various functions to manipulate the pagination state.
 */
declare function usePaginationQuery<Schema extends GenericSchema, Table extends Record<string, unknown>, Result extends Record<string, unknown>>(query: PostgrestTransformBuilder<Schema, Table, Result[]> | null, config?: SWRInfiniteConfiguration & {
    pageSize?: number;
}): UsePaginationQueryReturn<Result>;

/**
 * The return type of `useQuery` for `.single()` record results
 */
type UseQuerySingleReturn<Result> = Omit<SWRResponse<PostgrestSingleResponse<Result>['data'], PostgrestError$1>, 'mutate'> & Pick<SWRResponse<PostgrestSingleResponse<Result>, PostgrestError$1>, 'mutate'> & Pick<PostgrestSingleResponse<Result>, 'count'>;
/**
 * The return type of `useQuery` for `.maybeSingle()` queries
 */
type UseQueryMaybeSingleReturn<Result> = Omit<SWRResponse<PostgrestMaybeSingleResponse<Result>['data'], PostgrestError$1>, 'mutate'> & Pick<SWRResponse<PostgrestMaybeSingleResponse<Result>, PostgrestError$1>, 'mutate'> & Pick<PostgrestMaybeSingleResponse<Result>, 'count'>;
/**
 * The default return type of `useQuery` queries
 */
type UseQueryReturn<Result> = Omit<SWRResponse<PostgrestResponse<Result>['data'], PostgrestError$1>, 'mutate'> & Pick<SWRResponse<PostgrestResponse<Result>, PostgrestError$1>, 'mutate'> & Pick<PostgrestResponse<Result>, 'count'>;
/**
 * The return type of `useQuery` for any type of result
 */
type UseQueryAnyReturn<Result> = Omit<SWRResponse<AnyPostgrestResponse<Result>['data'], PostgrestError$1>, 'mutate'> & Pick<SWRResponse<AnyPostgrestResponse<Result>, PostgrestError$1>, 'mutate'> & Pick<AnyPostgrestResponse<Result>, 'count'>;
/**
 * Perform a postgrest query using `useSWR`.
 *
 * @param {PromiseLike<PostgrestSingleResponse<Result>> | null} query - The query to perform
 * @param {SWRConfiguration} [config] - The configuration for `useSWR`
 * @returns {UseQuerySingleReturn<Result>} - The query result
 */
declare function useQuery<Result>(query: PromiseLike<PostgrestSingleResponse<Result>> | null, config?: SWRConfiguration): UseQuerySingleReturn<Result>;
/**
 * Perform a postgrest query using `useSWR`.
 *
 * @param {PromiseLike<PostgrestMaybeSingleResponse<Result>> | null} query - The query to perform
 * @param {SWRConfiguration} [config] - The configuration for `useSWR`
 * @returns {UseQueryMaybeSingleReturn<Result>} - The query result
 */
declare function useQuery<Result>(query: PromiseLike<PostgrestMaybeSingleResponse<Result>> | null, config?: SWRConfiguration): UseQueryMaybeSingleReturn<Result>;
/**
 * Perform a postgrest query using `useSWR`.
 *
 * @param {PromiseLike<PostgrestResponse<Result>> | null} query - The query to perform
 * @param {SWRConfiguration} [config] - The configuration for `useSWR`
 * @returns {UseQueryReturn<Result>} - The query result
 */
declare function useQuery<Result>(query: PromiseLike<PostgrestResponse<Result>> | null, config?: SWRConfiguration): UseQueryReturn<Result>;

/**
 * Options for the useSubscriptionQuery hook.
 */
type UseSubscriptionQueryOpts<S extends GenericSchema, T extends GenericTable, Relationships, Q extends string = '*', R = GetResult<S, T['Row'], Relationships, Q extends '*' ? '*' : Q>> = PostgrestMutatorOpts<T['Row']> & MutatorOptions & {
    /**
     * A callback that will be called whenever a realtime event occurs for the given channel.
     * The callback will receive the event payload with an additional "data" property, which will be
     * the affected row of the event (or a modified version of it, if a select query is provided).
     */
    callback?: (event: RealtimePostgresChangesPayload<T['Row']> & {
        data: T['Row'] | R;
    }) => void | Promise<void>;
};
/**
 * A hook for subscribing to realtime Postgres events on a given channel.
 *
 * The subscription will automatically update the cache for the specified table in response
 * to incoming Postgres events, and optionally run a user-provided callback function with the
 * event and the updated data.
 *
 * This hook works by creating a Supabase Realtime channel for the specified table and
 * subscribing to Postgres changes on that channel. When an event is received, the hook
 * fetches the updated data from the database (using a `select` query generated from the cache
 * configuration), and then updates the cache accordingly.
 *
 * @param client - The Supabase client instance.
 * @param channelName - The name of the channel to subscribe to.
 * @param filter - The filter object to use when listening for changes.
 * @param primaryKeys - An array of the primary keys for the table being listened to.
 * @param query - An optional PostgREST query to use when selecting data for an event.
 * @param opts - Additional options to pass to the hook.
 * @returns An object containing the RealtimeChannel and the current status of the subscription.
 */
declare function useSubscriptionQuery<S extends GenericSchema, T extends GenericTable, Relationships, Q extends string = '*', R = GetResult<S, T['Row'], Relationships, Q extends '*' ? '*' : Q>>(client: SupabaseClient | null, channelName: string, filter: Omit<RealtimePostgresChangesFilter<`${REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.ALL}`>, 'table'> & {
    table: string;
}, primaryKeys: (keyof T['Row'])[], query?: QueryWithoutWildcard<Q> | null, opts?: UseSubscriptionQueryOpts<S, T, Relationships, Q, R>): {
    channel: RealtimeChannel | undefined;
    status: string | undefined;
};

/**
 * Options for `useSubscription` hook.
 */
type UseSubscriptionOpts<T extends GenericTable> = PostgrestMutatorOpts<T['Row']> & MutatorOptions & {
    /**
     * A callback that will be invoked whenever a new change event is received.
     *
     * @param event - The change event payload.
     * @returns Optionally returns a Promise.
     */
    callback?: (event: RealtimePostgresChangesPayload<T['Row']>) => void | Promise<void>;
};
/**
 * A custom React hook for subscribing to a Supabase Realtime subscription.
 *
 * @param channel - The Realtime subscription channel to listen to.
 * @param filter - The filter to apply on the table. Must include the table name.
 * @param primaryKeys - An array of primary key column names for the table.
 * @param opts - Additional options for the hook.
 * @returns An object containing the subscription status.
 */
declare function useSubscription<T extends GenericTable>(client: SupabaseClient | null, channelName: string, filter: Omit<RealtimePostgresChangesFilter<`${REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.ALL}`>, 'table'> & {
    table: string;
}, primaryKeys: (keyof T['Row'])[], opts?: UseSubscriptionOpts<T>): {
    status: string | undefined;
};

export { DecodedSWRKey, GetInputType, GetReturnType, INFINITE_KEY_PREFIX, INFINITE_PREFIX, KEY_PREFIX, KEY_SEPARATOR, Operation, POSTGREST_FILTER_KEY_PREFIX, SWRInfinitePaginationPostgrestResponse, SWRInfiniteScrollPostgrestResponse, UseInfiniteQueryReturn, UseInfiniteScrollQueryReturn, UsePaginationQueryReturn, UsePostgrestSWRMutationOpts, UseQueryAnyReturn, UseSubscriptionOpts, UseSubscriptionQueryOpts, createKeyGetter, decode, encode, infiniteMiddleware, middleware, useDeleteItem, useDeleteMutation, useInfiniteQuery, useInfiniteScrollQuery, useInsertMutation, usePaginationQuery, usePostgrestFilterCache, useQueriesForTableLoader, useQuery, useSubscription, useSubscriptionQuery, useUpdateMutation, useUpsertItem, useUpsertMutation };
